"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.desc = exports.command = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _tmp = _interopRequireDefault(require("tmp"));

var _ora = _interopRequireDefault(require("ora"));

var _path = _interopRequireDefault(require("path"));

var _yargs = _interopRequireDefault(require("yargs"));

var _ordinal = _interopRequireDefault(require("ordinal"));

var _saucelabs = _interopRequireDefault(require("saucelabs"));

var _changeCase = require("change-case");

var _runner = _interopRequireDefault(require("../runner"));

var _utils = require("../utils");

var _constants = require("../constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const command = 'run [params...] <site>';
exports.command = command;
const desc = 'Run performance tests on any website.';
exports.desc = desc;
const builder = _constants.RUN_CLI_PARAMS;
exports.builder = builder;

const handler = async argv => {
  const config = (0, _utils.getConfig)(argv);
  const username = config.user || process.env.SAUCE_USERNAME;
  const accessKey = config.key || process.env.SAUCE_ACCESS_KEY;
  const jobName = (0, _utils.getJobName)(config);
  const buildName = config.build || `${jobName} - ${new Date().toString()}`;
  const budget = config ? config.budget : null;
  const metrics = budget ? (0, _utils.getBudgetMetrics)(budget) : (0, _utils.getMetricParams)(config);
  const jankinessScore = (0, _utils.getJankinessParam)(argv, budget);
  /**
   * check if username and access key are available
   */

  if (!username || !accessKey) {
    _yargs.default.showHelp(); // eslint-disable-next-line no-console


    console.error(_constants.ERROR_MISSING_CREDENTIALS);
    return process.exit(1);
  }

  const status = (0, _ora.default)(`Start performance test run with user ${username} on page ${config.site}...`).start();
  const logDir = config.logDir ? _path.default.resolve(process.cwd(), config.logDir) : _tmp.default.dirSync().name;
  /**
   * check if job already exists
   */

  const user = new _saucelabs.default({
    user: username,
    key: accessKey,
    region: config.region,
    headers: {
      'User-Agent': _constants.SPEEDO_API_UA
    }
  });
  /**
   * find if job already exists
   */

  let job = null;

  try {
    job = await user.listJobs(username, {
      name: jobName,
      limit: 10
    });
    status.succeed();
  } catch (e) {
    status.fail(`Couldn't fetch job: ${e.stack}`);
    return process.exit(1);
  }
  /**
   * start Sauce Connect if not done by user
   */


  let tunnelProcess;

  if (config.tunnelIdentifier) {
    status.start(`Checking for Sauce Connect tunnel with identifier "${config.tunnelIdentifier}"`);

    try {
      tunnelProcess = await (0, _utils.startTunnel)(user, accessKey, logDir, config.tunnelIdentifier);

      if (tunnelProcess) {
        status.text = `Started Sauce Connect tunnel with identifier "${config.tunnelIdentifier}"`;
      }

      status.succeed();
    } catch (err) {
      status.fail(`Problem setting up Sauce Connect: ${err.stack}`);
      return process.exit(1);
    }
  }
  /**
   * create baseline if not enough tests have been executed
   */


  if (job.jobs.length < _constants.REQUIRED_TESTS_FOR_BASELINE_COUNT) {
    status.start(`Couldn't find baseline for job with name "${jobName}", creating baseline...`);
    const testCnt = _constants.REQUIRED_TESTS_FOR_BASELINE_COUNT - job.jobs.length;
    await Promise.all([...Array(testCnt)].map(() => (0, _runner.default)(username, accessKey, config, jobName, undefined, logDir)));
    status.succeed();
  }
  /**
   * run single test
   */


  status.start('Run performance test...');
  let {
    result,
    sessionId,
    benchmark,
    userAgent,
    jankinessResult
  } = await (0, _runner.default)(username, accessKey, config, jobName, buildName, logDir, jankinessScore);
  /**
   * retry performance test
   */

  const retriedJobs = [];

  if (result.result !== 'pass' && config.retry) {
    for (let retry = 1; retry <= config.retry; ++retry) {
      retriedJobs.push(sessionId);
      status.text = `Run performance test (${(0, _ordinal.default)(retry)} retry)...`;
      const retriedResult = await (0, _runner.default)(username, accessKey, config, jobName, buildName, logDir, jankinessScore);
      result = retriedResult.result;
      sessionId = retriedResult.sessionId;
      jankinessResult = retriedResult.jankinessResult;
      /**
       * continue command if job has finally passed
       */

      if (result.result === 'pass') {
        break;
      }
    }
  }

  status.succeed();
  /**
   * wait until job completes
   */

  try {
    status.start('Wait for job to finish...');
    await (0, _utils.waitFor)(() => user.getJob(username, sessionId),
    /* istanbul ignore next */
    jobDetails => jobDetails.status === 'complete');
    status.succeed();
  } catch (e) {
    status.fail(e.message);
    return process.exit(1);
  }
  /**
   * download performance logs
   */


  let performanceLog;

  try {
    status.start('Download performance logs...');
    const perfMetrics = await (0, _utils.waitFor)(() => user.getPerformanceMetricsByJobId(sessionId),
    /* istanbul ignore next */
    performanceMetrics => performanceMetrics.items.length !== 0, 'Couldn\'t receive any performance metrics');
    performanceLog = perfMetrics.items.map(item => ({
      sessionId: item.job_id,
      url: item.page_url,
      orderIndex: item.order_index,
      loaderId: item.loader_id,
      metrics: Object.entries(item.metric_data).reduce((obj, [metricName, metricValue]) => {
        obj[(0, _changeCase.camelCase)(metricName)] = metricValue;
        return obj;
      }, {})
    }));
    /**
     * store data log dir
     */

    _fs.default.writeFileSync(_path.default.join(logDir, 'performance.json'), JSON.stringify(performanceLog, null, 4));

    status.succeed();
  } catch (e) {
    status.fail(`Couldn't download performance results due to: ${e.stack}`);
    return process.exit(1);
  }
  /**
   * download trace file if requested
   */


  const loaderId = performanceLog[0].loaderId;

  if (config.traceLogs) {
    status.start('Download trace logs...');

    try {
      await user.downloadJobAsset(sessionId, `_tracelog_${loaderId}.json.gz`, _path.default.join(logDir, 'trace.json'));
      status.succeed();
    } catch (e) {
      status.fail(`Couldn't fetch trace logs: ${e.stack}`);
      status.stopAndPersist({
        text: 'continuing ...'
      });
    }
  }
  /**
   * update job if performance check fails
   */


  try {
    status.start('Updating job status...');
    await user.updateJob(username, sessionId, {
      passed: result.result === 'pass'
    });
    status.succeed();
  } catch (e) {
    status.fail(`Couldn't update job due to: ${e.stack}`);
    status.stopAndPersist({
      text: 'continuing ...'
    });
  }
  /**
   * stop tunnel if one was created in the beginning
   */


  if (tunnelProcess) {
    status.start('Stopping Sauce Connect tunnel...');
    await new Promise(resolve => {
      const shutdownTimeout = setTimeout(() => {
        status.warn('Sauce Connect shutdown timedout, you may still have tunnels lingering around');
        return resolve();
      }, _constants.TUNNEL_SHUTDOWN_TIMEOUT);
      return tunnelProcess.close(() => {
        clearTimeout(shutdownTimeout);
        status.succeed();
        resolve();
      });
    });
  }

  status.stopAndPersist({
    text: `Stored performance logs in ${logDir}`,
    symbol: '📃'
  });

  if (budget) {
    status.stopAndPersist({
      text: 'Asserting against performance budget of config file',
      symbol: '🧐 '
    });
    const preparedBudget = (0, _utils.prepareBudgetData)(budget);

    for (const [metric, [{
      l: lowerLimit,
      u: upperLimit
    }]] of Object.entries(preparedBudget)) {
      const actual = performanceLog[0].metrics[metric];
      /**
       * don't fail if - metric is within budget
       */

      if (actual <= upperLimit && actual >= lowerLimit || metric === _constants.JANKINESS_METRIC) {
        continue;
      }

      result.result = 'failed';
      result.details = result.details || {};
      result.details[metric] = {
        actual,
        upperLimit,
        lowerLimit
      };
    }
  }

  if (jankinessScore) {
    status.stopAndPersist({
      text: 'Asserting against jankinessScore',
      symbol: '🧐 '
    });

    if (!jankinessResult) {
      status.fail('Couldn\'t capture jankiness result');
    } else {
      const capturedJankinessScore = Math.round(jankinessResult.score * 100);
      const [{
        l: lowerLimit,
        u: upperLimit
      }] = jankinessScore[_constants.JANKINESS_METRIC];

      if (capturedJankinessScore > upperLimit || capturedJankinessScore < lowerLimit) {
        result.result = 'failed';
        result.details = result.details || {};
        result.details[_constants.JANKINESS_METRIC] = {
          actual: capturedJankinessScore,
          upperLimit,
          lowerLimit
        };
      }

      (0, _utils.printJankinessResult)(jankinessResult);
    }
  }

  (0, _utils.printResult)(result, performanceLog[0], metrics, config);
  const networkCondition = (0, _utils.getThrottleNetworkParam)(config);
  const runtimeSettings = [`- Network Throttling: ${networkCondition}`, `- CPU Throttling: ${config.throttleCpu}x`, `- CPU/Memory Power: ${benchmark} (${(0, _utils.getDeviceClassFromBenchmark)(benchmark)})`, `- User Agent: ${userAgent}`];
  status.stopAndPersist({
    text: `Runtime settings:\n${runtimeSettings.join('\n')}\n`,
    symbol: '⚙️ '
  });
  status.stopAndPersist({
    text: `Check out job at ${(0, _utils.getJobUrl)(config, sessionId)}`,
    symbol: '👀'
  });
  status.stopAndPersist({
    text: `Check out Lighthouse Report at ${(0, _utils.getLigthouseReportUrl)(config, sessionId, loaderId)}`,
    symbol: '📔'
  });
  /**
   * displayed retried jobs that failed
   */

  if (retriedJobs.length) {
    console.log( // eslint-disable-line no-console
    '\nFailed Performance Tests that were rerun:\n' + retriedJobs.map(id => (0, _utils.getJobUrl)(config, id)).join('\n'));
  }

  process.exit(result.result === 'pass' ? 0 : 1);
};

exports.handler = handler;